""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 13/01/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : returns angle in degrees

History : (13/01/2021) First version.
"""


import numpy as np

from SRPFITS.Fits.WCSPixelScale import WCSPixelScale


def WCSRotationDeg(WCS):
    scl = WCSPixelScale(WCS)
    xs = scl[0].value
    #
    ang = np.degrees(np.arcsin(WCS.pixel_scale_matrix[1][0]/xs))
    return ang


""" Utility functions and classes for SRP

Context : SRP
Module  : Photometry
Version : 1.1.0
Author  : Stefano Covino
Date    : 30/11/2034
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 
    
History : (15/02/2012) First version.
        : (30/11/2023) Zeropoint error. 
"""

import math
import numpy


def Counts2Mag (cnt, ecnt, zp=0.0, ezp=0.0):
    mag = -2.5*numpy.log10(numpy.array(cnt)) + zp
    emag = (2.5/math.log(10))*(numpy.array(ecnt)/numpy.array(cnt))
    emag = numpy.sqrt(emag**2 + ezp**2)
    return mag, emag


""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/01/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : returns pixel scales in degrees

History : (14/01/2021) First version.
"""


import numpy as np

def WCSPixelScale(WCS):
    scl = WCS.proj_plane_pixel_scales()
    return scl



""" Utility functions and classes for SRP

Context : SRP
Module  : PhotometryClass.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/10/2024
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (02/10/2024) First version.
"""

#import os
import numpy as np
from astropy.table import Table
import sep
from SRPFITS.Photometry.ApyPhot import ApyPhot
#from SRPFITS.Photometry.DaoPhot import DaoPhot
from SRPFITS.Fits.GetData import GetData
#from .GetHeader import GetHeader
#from SRPFITS.Fits.GetWCS import GetWCS
#from .GetHeaderValue import GetHeaderValue
#from . import FitsConstants as FitsConstants
#from SRPFITS.GetFWHM import GetFWHM
from SRPFITS.Photometry.Counts2Mag import Counts2Mag
#from SRPFITS.Frames.SourceObjectsClass import SourceObjects
#from SRPFITS.Frames.SexObjectClass import SexObjects
#from SRPFITS.Frames.DAOObjectClass import DAOObjects
#from SRPFITS.Frames.Pixel2WCS import Pixel2WCS
#from astropy import log
#log.setLevel('WARNING')

class FitsPhotometry:
    def __init__ (self, fitsfile, objlist, exptime=1., airmass=1., extcoeff=0., zeropoint=(25.0,0.0), level=3):
        self.Fitsfile = fitsfile
        self.ObjList = objlist
        self.Exptime = exptime
        self.Airmass = airmass
        self.ExtCoeff = extcoeff
        self.Zeropoint = zeropoint
        self.Level = level

    def GetImageData (self, extension=0):
        try:
            self.Data = GetData (self.Fitsfile, extension)[0]
            return True
        except:
            return False
        
    def GetObjPos (self):
        try:
            iftab = Table.read(self.ObjList,format='ascii')
            self.X = iftab.columns[1]
            self.Y = iftab.columns[2]
            return True
        except:
            return False
    
    
    def SexPhotometry (self):
        # From here: https://python.hotexamples.com/it/examples/sep/-/set_extract_pixstack/python-set_extract_pixstack-function-examples.html
        data = self.Data
        #
        try:
            bkg = sep.Background(data)
        except ValueError:
            data = data.byteswap(True).newbyteorder()
            bkg = sep.Background(data)
        bkg.subfrom(data)
        #
        sources = sep.extract(data, self.Level * bkg.globalrms)
        #
        sources = Table(sources)
        # Calculate the ellipticity
        sources['ellipticity'] = 1.0 - (sources['b'] / sources['a'])
        # Fix any value of theta that are invalid due to floating point rounding
        # -pi / 2 < theta < pi / 2
        sources['theta'][sources['theta'] > (np.pi / 2.0)] -= np.pi
        sources['theta'][sources['theta'] < (-np.pi / 2.0)] += np.pi
        # Calculate the kron radius
        kronrad, krflag = sep.kron_radius(data, sources['x'], sources['y'],
            sources['a'], sources['b'], sources['theta'], 6.0)
        sources['flag'] |= krflag
        sources['kronrad'] = kronrad
        # Calculate the equivilent of flux_auto
        flux, fluxerr, flag = sep.sum_ellipse(data, sources['x'], sources['y'],
            sources['a'], sources['b'], np.pi / 2.0, 2.5 * kronrad, subpix=1, err=self.Level * bkg.globalrms)
        sources['flux'] = flux
        sources['fluxerr'] = fluxerr
        sources['flag'] |= flag
        # Calculate the FWHMs of the stars:
        fwhm = 2.0 * (np.log(2) * (sources['a'] ** 2.0 + sources['b'] ** 2.0)) ** 0.5
        sources['fwhm'] = fwhm
        # Cut individual bright pixels. Often cosmic rays
        sources = sources[fwhm > 1.0]
        # Measure the flux profile
        flux_radii, flag = sep.flux_radius(data, sources['x'], sources['y'],
            6.0 * sources['a'], [0.25, 0.5, 0.75], normflux=sources['flux'], subpix=5)
        sources['flag'] |= flag
        sources['fluxrad25'] = flux_radii[:, 0]
        sources['fluxrad50'] = flux_radii[:, 1]
        sources['fluxrad75'] = flux_radii[:, 2]
        # Calculate the windowed positions
        sig = 2.0 / 2.35 * sources['fluxrad50']
        xwin, ywin, flag = sep.winpos(data, sources['x'], sources['y'], sig)
        sources['flag'] |= flag
        sources['xwin'] = xwin
        sources['ywin'] = ywin
        # Calculate the average background at each source
        bkgflux, fluxerr, flag = sep.sum_ellipse(bkg.back(), sources['x'], sources['y'],
            sources['a'], sources['b'], np.pi / 2.0, 2.5 * sources['kronrad'], subpix=1)
        background_area = (2.5 * sources['kronrad']) ** 2.0 * sources['a'] * sources['b'] * np.pi # - masksum
        sources['background'] = bkgflux
        sources['background'][background_area > 0] /= background_area[background_area > 0]
        # Update the catalog to match fits convention instead of python array convention
        sources['x'] += 1.0
        sources['y'] += 1.0
        sources['xpeak'] += 1
        sources['ypeak'] += 1
        sources['xwin'] += 1.0
        sources['ywin'] += 1.0
        sources['theta'] = np.degrees(sources['theta'])
        #
        # Exptime
        flux, fluxerr = flux/self.Exptime, fluxerr/self.Exptime
        # Convert to magnitudes
        # Zero point
        mag,magerr = Counts2Mag(flux,fluxerr,self.Zeropoint[0],self.Zeropoint[1])
        # Airmass
        mag = mag - self.ExtCoeff*self.Airmass
        #
        sources['mag'] = mag
        sources['emag'] = magerr
        self.SexResults = sources
        return Table([sources['x'],sources['y'],flux,fluxerr,mag,magerr],names=('X','Y','Flux','eFlux','Mag','eMag'))
        
        
    def ApyPhotometry (self, rds=(5,10,15),backgr=True,gain=1.,ron=0.):
            flux,fluxerr = ApyPhot (self.X,self.Y,self.Data,rds,backgr,gain,ron)
            # Exptime
            flux, fluxerr = flux/self.Exptime, fluxerr/self.Exptime
            # Zero point
            mag, emag = Counts2Mag (flux,fluxerr,self.Zeropoint[0],self.Zeropoint[1])
            # Airmass
            mag = mag - self.ExtCoeff*self.Airmass
            return Table([self.X,self.Y,flux,fluxerr,mag,emag],names=('X','Y','Flux','eFlux','Mag','eMag'))

                       
    #def DaoPhotometry (self, rds=(5,10,15),backgr=True,gain=1.,ron=0.,skyalg='mmm'):
    #        flux,fluxerr = DaoPhot (self.X,self.Y,self.Data,rds,backgr,gain,ron,skyalg)
    #        return Table([self.X,self.Y,flux,fluxerr],names=('X','Y','Flux','eFlux'))




""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.9.7
Author  : Stefano Covino
Date    : 02/03/2021
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/05/2010) First version.
        : (15/07/2010) WCS added.
        : (24/08/2010) Minor correction and source list sorting.
        : (25/08/2010) Frame size.
        : (30/08/2010) Fits Headers.
        : (04/09/2010) New importing rules.
        : (27/09/2010) Eclipse sources added.
        : (28/09/2010) Named changed from ObjList to FitsImage and adapted to native source class. 
        : (29/09/2010) Naming rationalized.
        : (04/10/2010) Possibility to remove stars close to frame border.
        : (25/10/2010) Basic statistics for frames.
        : (19/08/2013) Possiblity to read different extensions and original BITPIX saved.
        : (16/10/2013) Frame center in pixel.
        : (21/01/2014) Possibility to get statistixsw for just a subregion.
        : (27/10/2014) Sextractor source finding.
        : (16/05/2017) Minor update.
        : (26/05/2017) DAOPHOT source finding.
		: (30/11/2020) Minor GetWCS syntax correction.
        : (03/12/2020) More WCS corrections.
		: (11/12/2020) Again minor corrections.
        : (13/01/2020) Astropy WCS library.
		: (28/01/2021) Matt Hilton corrected the bugs of astLib.
        : (02/02/2021) Better management of astropy.log.
        : (02/03/2021) Better sorting.
"""

import os

import numpy

from .GetData import GetData
from .GetHeader import GetHeader
from SRPFITS.Fits.GetWCS import GetWCS
from .GetHeaderValue import GetHeaderValue
from . import FitsConstants as FitsConstants
from SRPFITS.GetFWHM import GetFWHM

from SRPFITS.Frames.SourceObjectsClass import SourceObjects
from SRPFITS.Frames.SexObjectClass import SexObjects
from SRPFITS.Frames.DAOObjectClass import DAOObjects
from SRPFITS.Frames.Pixel2WCS import Pixel2WCS
from astropy import log
log.setLevel('WARNING')

class FitsImage:
    def __init__ (self, fitsfile, extension=0):
        self.Name = fitsfile
        self.Header = GetHeader(fitsfile,extension)[0]
        self.Data = GetData(fitsfile,extension)[0]
        self.WCS = GetWCS(fitsfile,'astlib')[0]
        self.BITPIX = GetHeaderValue(fitsfile,'BITPIX',extension)[0]
        self.NAXIS = GetHeaderValue(fitsfile,'NAXIS',extension)[0]
        self.List = []
        self.NativeSourcesFlag = False
        self.DAOSourcesFlag = False
        self.SexSourcesFlag = False


    def Sources(self, threshold=5.0, filtsing=3):
        slist = SourceObjects(self.Name)
        slist.FindObjects(self.Data, threshold, filtsing)
        srclist = []
        for i in slist.ListEntries:
            srclist.append((i.X,i.Y))
        coolist = Pixel2WCS(self.Header,srclist,'astlib')
        for i,l in zip(slist.ListEntries,coolist):
            i.RA = l[0]
            i.DEC = l[1]
        self.List = slist.ListEntries
        self.NativeSourcesFlag = True
        self.DAOSourcesFlag = False
        self.SexSourcesFlag = False
        return len(self.List)
        
        
    def SortSourceList(self):
        if self.NativeSourcesFlag:
            self.List.sort()
            #self.List.reverse()
        elif self.DAOSourcesFlag:
            self.List.sort()
            #self.List.reverse()
        elif self.SexSourcesFlag:
            self.List.sort()
            #self.List.reverse()


    def GetFWHM (self):
        X = []
        Y = []
        for i in self.List:
            X.append(i.X)
            Y.append(i.Y)
        fwhml = GetFWHM(X,Y,self.Name)
        for i,l in zip(self.List,fwhml):
            i.FWHM = l
        return len(self.List)

        
    def GetFrameSizePix(self):
        base = FitsConstants.NAxis
        sizelist = []
        try:
            naxis = self.Header[base]
        except KeyError:
            return None
        for i in range(naxis):
            baseax = '%s%s' % (base,i+1)
            try:
                value = self.Header[baseax]
            except KeyError:
                value = None
            sizelist.append(value)
        return sizelist



    def SexSources(self,level=3.0):
        elist = SexObjects(self.Name,level)
        elist.FindSexObjects()
        srclist = []
        for i in elist.ListEntries:
            srclist.append((i.X,i.Y))
        try:
            coolist = Pixel2WCS(self.Header,srclist,'astlib' )
        except AttributeError:
            coolist = []
            for ii in range(len(srclist)):
                coolist.append((0.,0.))
        for i,l in zip(elist.ListEntries,coolist):
            i.RA = l[0]
            i.DEC = l[1]
        self.List = elist.ListEntries
        self.SexSourcesFlag = True
        self.DAOSourcesFlag = False
        self.NativeSourcesFlag = False
        return len(self.List)



    def DAOSources(self,level=3.0):
        elist = DAOObjects(self.Name,level)
        elist.FindDAOObjects()
        srclist = []
        for i in elist.ListEntries:
            srclist.append((i.X,i.Y))
        try:
            coolist = Pixel2WCS(self.Header,srclist,'astlib')
        except AttributeError:
            coolist = []
            for ii in range(len(srclist)):
                coolist.append((0.,0.))
        for i,l in zip(elist.ListEntries,coolist):
            i.RA = l[0]
            i.DEC = l[1]
        self.List = elist.ListEntries
        self.SexSourcesFlag = False
        self.DAOSourcesFlag = True
        self.NativeSourcesFlag = False
        return len(self.List)



    def CleanBorderSources(self,cleaningpercentage=1):
        framesize = self.GetFrameSizePix()
        avoidzone = (framesize[0]*cleaningpercentage/100.,framesize[1]*cleaningpercentage/100.)
        CleanList = []
        oldnobj = len(self.List)
        for entry in self.List:
            if (1+avoidzone[0] <= entry.X <= framesize[0]-avoidzone[0]) and (1+avoidzone[1] <= entry.Y <= framesize[1]-avoidzone[1]):
                CleanList.append(entry)
        self.List = CleanList
        return len(self.List),oldnobj
        
        
    def __str__(self):
        msg = ''
        # Native
        if self.NativeSourcesFlag:
            for i in range(len(self.List)):
                msg = msg + str(self.List[i])
        # daophot
        elif self.DAOSourcesFlag:
            for i in range(len(self.List)):
                msg = msg + str(self.List[i])
        # Sex
        elif self.SexSourcesFlag:
            for i in range(len(self.List)):
                msg = msg + str(self.List[i])
        return msg
    

    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (self.Name)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "x_col: 1\n"
        msg = msg + "y_col: 2\n"
        # Native
        if self.NativeSourcesFlag:
            msg = msg + "Id\tX\tY\tRA\tDEC\tNPix\tMag\tFWHM\n"
        # daophot
        elif self.DAOSourcesFlag:
            msg = msg + "Id\tX\tY\tSharpness\tRoundness1\tRoundness2\tNpix\tSky\tPeak\tFlux\tFWHM\tRA\tDEC\n"
        # Sex
        elif self.SexSourcesFlag:
            msg = msg + "Id\tX\tY\tNpix\tEllipticity\tFlux\tPeak\tFlag\tFWHM\tRA\tDEC\n"
        
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def GetStats(self,region=None):
        """
        region is a list of four pixel position (lower left is 1,1)
        They are leftx, bottomy, rightx, uppery.
        """
        if region == None or len(region) != 4:
            mean = numpy.mean(self.Data)
            std = numpy.std(self.Data)
            median = numpy.median(self.Data)
            max = numpy.max(self.Data)
        else:
            try:
                mean = numpy.mean(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
                std = numpy.std(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
                median = numpy.median(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
                max = numpy.max(self.Data[region[1]+1:region[3]+1,region[0]+1:region[2]+1])
            except ValueError:
                return numpy.nan,numpy.nan,numpy.nan,numpy.nan
        #
        return mean,std,median,max



    def GetFrameCenterPix (self):
        fs = self.GetFrameSizePix()
        return int(fs[0]/2.), int(fs[1]/2.)




""" Utility functions and classes for SRP

Module  : Frames.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 07/03/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (31/08/2012) First version.
        : (15/05/2017) Minor update.
        : (07/03/2020) astropy wcs tools.
"""

from SRPFITS.Fits.GetWCS import GetWCS


def getCenterRADEC (fitsfile):
    wcs = GetWCS(fitsfile)[0]
    if wcs != None:
        xpix, ypix = wcs.pixel_shape
        return wcs.all_pix2world(xpix/2.,ypix/2.,1,ra_dec_order=True)
    else:
        return None



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.1.3
Author  : Stefano Covino
Date    : 06/07/2021
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2010) First version.
        : (13/10/2010) Quicker triangle scan.
        : (26/08/2013) More control on accepted matches.
        : (14/03/2017) Possibility to remove not useful iterations 
            and porting to SRP.FITS.
        : (06/07/2021) SREPSTATS.
"""

import math

from SRP.SRPMath.TriangleClass import Triangle
from SRP.SRPMath.CartesianCoordAmpRotoTranslation import CartesianCoordAmpRotoTranslation
from SRPSTATS.CoordDistanceMinimization import CoordDistanceMinimization
from SRP.SRPMath.AngleRange import AngleRange



def TriangleMatch (reflist,objlist,angtol=0.5,disttol=5.,TrueSize=None,TrueAng=None):
#    print objlist
#    print reflist
    if len(reflist) >=3 and len(objlist) >=3:
        #
        if len(reflist) >= 5:
            reqmtch = 5
        else:
            reqmtch = len(reflist)
        #
        for i in range(len(reflist)):
            for ii in range(i,len(reflist)):
                for iii in range(ii,len(reflist)):
                    for l in range(len(objlist)):
                        for ll in range(l,len(objlist)):
                            for lll in range(ll,len(objlist)):
                                if i != ii and ii != iii and i != iii and l != ll and ll != lll and l != lll:
                                    #print i,ii,iii,l,ll,lll
                                    triref = Triangle(reflist[i],reflist[ii],reflist[iii],angtol)
                                    triobj = Triangle(objlist[l],objlist[ll],objlist[lll],angtol)
                                    if not (triobj.IsSymmTriangle() or triref.IsSymmTriangle()):
                                        # cannot work with symmetric triangles
                                        # Select not too small or elongated triangles
                                        if min(triobj.Sizes) >= 10.0 and min(triobj.Angles) > 10.0 and triobj.Commensurabili(triref):
                                            if triobj.Rotable(triref):
                                                # with a good candidate compute pairing data
                                                sf = triobj.SizeFactor(triref)
                                                rf = triobj.RotationFactor(triref)
                                                sh = triobj.ShiftFactor(triref)
                                                if TrueSize != None:
                                                    if TrueSize/sf > 1.1 or TrueSize/sf < 0.9:
                                                        continue
                                                if TrueAng != None:
                                                    if abs(AngleRange(TrueAng)-AngleRange(rf)) > 10.:
                                                        continue
                                                #print i,ii,iii,l,ll,lll
                                                #print ("sf,rf,sz", sf,rf,sh)
                                                #print "objs",triobj.Vertici[0],triobj.Vertici[1],triobj.Vertici[2]
                                                #print "refs",triref.Vertici[0],triref.Vertici[1],triref.Vertici[2]
                                                #print "obj-ang", triobj.Angles
                                                #print "ref-ag", triref.Angles
                                                #print "obj-siz", triobj.Sizes
                                                #print "ref-siz", triref.Sizes
                                                #print "ccns",triobj.ConnVert
                                                # find acceptable error
                                                tempinp = []
                                                tempout = []
                                                for zz in triobj.ConnVert:
                                                    tempinp.append(triobj.Vertici[zz[0]].Coord)
                                                    tempout.append(triref.Vertici[zz[1]].Coord)
                                                res = CoordDistanceMinimization(tempout,tempinp,[sh[0],sh[1],rf,1./sf])   
                                                # acceptable tolerance for match 
                                                minassdist = disttol*res[1]
                                                #print minassdist, res
                                                #print
                                                #    
                                                #minassdist = 0
                                                #for zz in triobj.ConnVert:
                                                #    miecord = CartesianCoordAmpRotoTranslation(triobj.Vertici[zz[0]].Coord,sh,rf,1./sf)
                                                #    miecorddist = math.sqrt((miecord[0]-triref.Vertici[zz[1]].Coord[0])**2 + (miecord[1]-triref.Vertici[zz[1]].Coord[1])**2)
                                                #    minassdist = minassdist + miecorddist 
                                                #    #print miecorddist,triobj.Vertici[zz[0]].Coord, miecord, triref.Vertici[zz[1]].Coord
                                                #minassdist = minassdist*5./3.
                                                # with a basic solution try to find a general one
                                                # pair points
                                                nowinp = []
                                                nowout = []
                                                assobj = [False for c in reflist]
                                                for pp in range(len(objlist)):
                                                    # convert coordinates from obj to ref
                                                    miecord = CartesianCoordAmpRotoTranslation(objlist[pp],(res[0][0],res[0][1]),res[0][2],res[0][3])
                                                    #print "miecord",miecord
                                                    mindist = 1e12
                                                    objmem = 0
                                                    for qq in range(len(reflist)):
                                                        #print pp, qq, assobj[qq]
                                                        #print "obj",objlist[pp],"ref",reflist[qq]
                                                        # if point is not assigned and close enough to a frame point go on
                                                        if not assobj[qq]:
                                                            mdist = math.sqrt((miecord[0]-reflist[qq][0])**2 + (miecord[1]-reflist[qq][1])**2)
                                                            if mdist < mindist:
                                                                mindist = mdist
                                                                objmem = qq
                                                    if mindist <= minassdist:
                                                        assobj[objmem] = True
                                                        nowinp.append(pp)
                                                        nowout.append(objmem)
                                                    #print "dist",mindist,minassdist,objmem
                                                # now we check how many associated points we have
                                                #print len(nowout),reqmtch
                                                if len(nowout) >= reqmtch:
                                                    return nowinp,nowout,res
                                            else:
                                                pass
#                                                print "Not rotable."
    return None
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 2.1.0
Author  : Stefano Covino
Date    : 30/11/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (15/07/2010) First version.
        : (20/12/2010) Possibility of badly formatted ECS header included.
        : (23/08/2013) NUMPY_MODE in wcs.
        : (16/12/2014) Manage wrong FITS headers.
        : (07/03/2020) astropy.wcs
		: (20/11/2020) astLib WCS again available.
"""



from astropy.wcs import WCS
from SRPFITS.Fits import FitsConstants
from SRPFITS.Fits.IsFits import IsFits
from .GetHeader import GetHeader
import astLib.astWCS as aw



def GetWCS (fitsfile,mode='apy'):
    if mode == 'apy':
        if IsFits(fitsfile):
            wcs = WCS(fitsfile)
            return wcs,FitsConstants.FitsWCSFound
        else:
            return None,FitsConstants.FitsFileNotFound
    else:
        heder = GetHeader(fitsfile)
        if heder[1] == FitsConstants.FitsHeaderFound:
            try:
                wcs = aw.WCS(heder[0],mode='pyfits')
            except IOError:
                return None,FitsConstants.FitsFileNotFound
            except ValueError:
                return None,FitsConstants.FitsWCSNotKnown
            return wcs,FitsConstants.FitsWCSFound
        else:
            return None,FitsConstants.FitsFileNotFound
